// EnvVector.h

#ifndef __EnvVector_h__
#define __EnvVector_h__

#include <vector>

class Gene;

class EnvVector
{
public:
	EnvVector();
	EnvVector( int size, float initialValue );
	~EnvVector();

	EnvVector & operator = (const EnvVector& other);

	int length() const;
	void modify( const Gene * pGene );
	void add( const Gene * pGene );
	void divide( float divisor );
	vector<float> & getValues();

	friend ostream& operator << (ostream& os, const EnvVector& id);


private:
	// methods
	void getModifier( const Gene * pGene, EnvVector * pModifier, int * pStartPos );

	void generateFunction0( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction1( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction2( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction3( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction4( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction5( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction6( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );
	void generateFunction7( vector<float> & vec, int repeat, int patternLength, int patternLengthFinal );

	void operatorMult( int startPos, EnvVector & modifier );
	void operatorUnion( int startPos, EnvVector & modifier );
	void operatorIntersection( int startPos, EnvVector & modifier );
	void operatorCompUnion( int startPos, EnvVector & modifier );
	void operatorCompIntersection( int startPos, EnvVector & modifier );
	void operatorAvg( int startPos, EnvVector & modifier );
	void operatorCompAvg( int startPos, EnvVector & modifier );
	void operatorAUnionCompB( int startPos, EnvVector & modifier );
	void operatorAIntersectionCompB( int startPos, EnvVector & modifier );
	void operatorCompAUnionB( int startPos, EnvVector & modifier );
	void operatorCompAIntersectionB( int startPos, EnvVector & modifier );
	void operatorCompMult( int startPos, EnvVector & modifier );
	void operatorCompAUnionMult( int startPos, EnvVector & modifier );
	void operatorCompAIntersectionMult( int startPos, EnvVector & modifier );
	void operatorCompBUnionMult( int startPos, EnvVector & modifier );
	void operatorCompBIntersectionMult( int startPos, EnvVector & modifier );

	// data members
	vector<float> m_Values;
};


inline EnvVector::EnvVector()
{
}


inline EnvVector::EnvVector( int size, float initialValue )
:
	m_Values( size, initialValue )
{
}


inline EnvVector::~EnvVector()
{
	//cout << "EnvVector destructor called" << endl;
}


inline EnvVector & EnvVector::operator = (const EnvVector& other)
{
	m_Values = other.m_Values;
	return (*this);
}


inline int EnvVector::length() const
{
	return m_Values.size();
}


inline vector<float> & EnvVector::getValues()
{
	return m_Values;
}


#endif
