// Gene.cc

#include "Gene.h"
#include "Simulator.h"
#include <iostream>


Gene::Gene( bool bRandomCreate /* = false */ )
:
	m_Function(0),
	m_Partition(0),
	m_Spread(0),
	m_Repeat(0),
	m_Operator(0)
{
	if (bRandomCreate)
		randomCreate();
}


Gene::Gene( int function, int partition, int spread, int repeat, int op )
:
	m_Function( function ),
	m_Partition( partition ),
	m_Spread( spread ),
	m_Repeat( repeat ),
	m_Operator( op )
{
	if (m_Function < 0)
		m_Function = 0;
	else if (m_Function >= NUM_FUNCTIONS)
		m_Function = NUM_FUNCTIONS-1;
	
	if (m_Partition < 0)
		m_Partition = 0;
	else if (m_Partition >= NUM_PARTITIONS)
		m_Partition = NUM_PARTITIONS-1;
	
	if (m_Spread < 0)
		m_Spread = 0;
	else if (m_Spread > MAX_SPREAD)
		m_Spread = MAX_SPREAD;
	
	if (m_Repeat < 0)
		m_Repeat = 0;
	else if (m_Repeat > MAX_REPEAT)
		m_Repeat = MAX_REPEAT;
	
	if (m_Operator < 0)
		m_Operator = 0;
	else if (m_Operator >= NUM_OPERATORS)
		m_Operator = NUM_OPERATORS-1;	
}


void Gene::randomCreate( bool bDisturbanceGene /* = false */ )
{
	m_Function = Simulator::m_RNG.randInt() % NUM_FUNCTIONS;
	m_Partition = Simulator::m_RNG.randInt() % NUM_PARTITIONS;
	
	if (bDisturbanceGene && Simulator::ENVIRONMENT_DISTURBANCE_RESTRICTED)
		m_Spread = Simulator::m_RNG.randInt() % (Simulator::ENVIRONMENT_DISTURBANCE_MAX_SPREAD + 1);
	else
		m_Spread = Simulator::m_RNG.randInt() % (MAX_SPREAD + 1);
	
	m_Repeat = Simulator::m_RNG.randInt() % (MAX_REPEAT + 1);
	m_Operator = Simulator::m_RNG.randInt() % NUM_OPERATORS;
}


void Gene::mutate( bool bDisturbanceGene /* = false */ )
{
	int mutand = Simulator::m_RNG.randInt() % 5;
	
	int delta = 1;
	if (mutand > 0 && mutand < 4)
		delta = 1 - (Simulator::m_RNG.randInt() % 3);
	
	switch (mutand)
	{
		case 0: m_Function = Simulator::m_RNG.randInt() % NUM_FUNCTIONS; break;
		case 1: m_Partition = (NUM_PARTITIONS + m_Partition + delta) % NUM_PARTITIONS; break;
		case 2: 
		{
			if (bDisturbanceGene && Simulator::ENVIRONMENT_DISTURBANCE_RESTRICTED)
				m_Spread = (Simulator::ENVIRONMENT_DISTURBANCE_MAX_SPREAD + 1 + m_Spread + delta) % (Simulator::ENVIRONMENT_DISTURBANCE_MAX_SPREAD + 1);
			else
				m_Spread = (MAX_SPREAD + 1 + m_Spread + delta) % (MAX_SPREAD + 1);
			break;
		}
		case 3: m_Repeat = (MAX_REPEAT + 1 + m_Repeat + delta) % (MAX_REPEAT + 1); break;
		case 4: m_Operator = Simulator::m_RNG.randInt() % NUM_OPERATORS; break;
		default:
		{
			cerr << "In method Gene::mutate. Unexpected mutand " << mutand << "!" << endl;
			exit( EXIT_FAILURE );
		}			
	}
}


ostream& operator << (ostream& os, Gene& id)
{
	os << id.m_Function << " " << id.m_Partition << " " << id.m_Spread << " " << id.m_Repeat << " " << id.m_Operator;
	return os;
}
